﻿using ChangePointSystem.Models;
using ChangePointSystem.Repository;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Reflection;
using System.Web.Mvc;

namespace ChangePointSystem.Util
{

    /// <summary>
    /// SelectListItemに変換する
    /// </summary>
    public static class SelectListUtil
    {
        /// <summary>
        /// タイプからSelectListItem
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        public static IList<SelectListItem> GetSelectList(Type type)
        {
            if (type == null)
            {
                throw new ArgumentNullException("type");
            }

            if (!IsValidForEnumHelper(type))
            {
                throw new ArgumentException(String.Format("{0} の型はサポートされていません", type.FullName), "type");
            }

            IList<SelectListItem> selectList = new List<SelectListItem>();

            Type checkedType = Nullable.GetUnderlyingType(type) ?? type;
            if (checkedType != type)
            {
                // Underlying type was non-null so handle Nullable<T>; ensure returned list has a spot for null
                selectList.Add(new SelectListItem { Text = String.Empty, Value = String.Empty, });
            }

            // Populate the list
            const BindingFlags BindingFlags =
                BindingFlags.DeclaredOnly | BindingFlags.GetField | BindingFlags.Public | BindingFlags.Static;
            foreach (FieldInfo field in checkedType.GetFields(BindingFlags))
            {
                // fieldValue will be an numeric type (byte, ...)
                object fieldValue = field.GetRawConstantValue();

                selectList.Add(new SelectListItem { Text = GetDisplayName(field), Value = fieldValue.ToString(), });
            }

            return selectList;
        }

        /// <summary>
        /// ラインマスタのSelectListItem
        /// </summary>
        /// <param name="list"></param>
        /// <returns></returns>
        public static IList<SelectListItem> GetLineSelectList()
        {
            LineRepository repository = new LineRepository();
            IList<SelectListItem> selectList = new List<SelectListItem>();
            foreach (m_line item in repository.FindAllLineList())
            {
                selectList.Add(new SelectListItem { Text = item.line_name, Value = item.line_code});
            }

            return selectList;
            
        }

        /// <summary>
        /// ラインマスタ(By上位組織)のSelectListItem
        /// </summary>
        /// <param name="list"></param>
        /// <returns></returns>
        public static IList<SelectListItem> GetLineByOrgSelectList(string userId)
        {
            LineRepository repository = new LineRepository();
            IList<SelectListItem> selectList = new List<SelectListItem>();
            foreach (m_line item in repository.FindLineListByUserId(userId))
            {
                selectList.Add(new SelectListItem { Text = item.line_name, Value = item.line_code });
            }

            return selectList;

        }

        /// <summary>
        /// 工程マスタのSelectListItem
        /// </summary>
        /// <param name="list"></param>
        /// <returns></returns>
        public static IList<SelectListItem> GetProcessSelectList(String lineCode)
        {
            ProcessRepository repository = new ProcessRepository();
            IList<SelectListItem> selectList = new List<SelectListItem>();
            if(String.IsNullOrEmpty(lineCode))
            {
                return selectList;
            }
            foreach (m_process item in repository.FindProcessListByLine(lineCode))
            {
                selectList.Add(new SelectListItem { Text = item.process_name, Value = item.process_code });
            }

            return selectList;

        }

        /// <summary>
        /// 4M区分のSelectListItem
        /// </summary>
        /// <param></param>
        /// <returns></returns>
        public static IList<SelectListItem> GetChangeTypeSelectList()
        {
            IList<SelectListItem> selectList = new List<SelectListItem>();
            
            foreach (var item in Const.CHANGE_TYPE_CODE_LABEL)
            {
                selectList.Add(new SelectListItem { Text = item.Value, Value = item.Key });
            }

            return selectList;
        }

        /// <summary>
        /// 対応状況のチェックボックス選択肢
        /// </summary>
        /// <param></param>
        /// <returns></returns>
        public static IList<SelectListItem> GetStatustCheckBoxList()
        {
            IList<SelectListItem> selectList = new List<SelectListItem>();

            foreach (var item in Const.DEAL_STATUS_CODE_LABEL)
            {
                selectList.Add(new SelectListItem { Text = item.Value, Value = item.Key });
            }

            return selectList;
        }

        /// <summary>
        /// 確認結果のSelectListItem
        /// </summary>
        /// <param></param>
        /// <returns></returns>
        public static IList<SelectListItem> GetConfirmResultSelectList()
        {
            IList<SelectListItem> selectList = new List<SelectListItem>();

            foreach (var item in Const.CONFIRM_RESULT_CODE_LABEL)
            {
                selectList.Add(new SelectListItem { Text = item.Value, Value = item.Key });
            }

            return selectList;
        }

        /// <summary>
        /// 組織コードに含まれるライン
        /// </summary>
        /// <param></param>
        /// <returns></returns>
        public static IList<SelectListItem> GetOrganizationLineSelectList()
        {
            LineRepository repository = new LineRepository();
            IList<SelectListItem> selectList = new List<SelectListItem>();
            // アクセス組織リスト取得
            List<String> organizationCodeList = StaticUtil.GetOrganizationList();
            foreach (m_line item in repository.FindOrganizationLineMap(organizationCodeList))
            {
                selectList.Add(new SelectListItem { Text = item.line_name, Value = item.line_code });
            }

            return selectList;
        }

        /// <summary>
        /// 前後区分のSelectListItem
        /// </summary>
        /// <param></param>
        /// <returns></returns>
        public static IList<SelectListItem> GetBeforeAfterSelectList()
        {
            IList<SelectListItem> selectList = new List<SelectListItem>();

            selectList.Add(new SelectListItem { Text = "前", Value = "1" });
            selectList.Add(new SelectListItem { Text = "後", Value = "2" });
            selectList.Add(new SelectListItem { Text = "その他", Value = "9" });

            return selectList;
        }

        /// <summary>
        /// 検索条件区分のSelectListItem
        /// </summary>
        /// <param></param>
        /// <returns></returns>
        public static IList<SelectListItem> GetSearchTypeSelectList()
        {
            IList<SelectListItem> selectList = new List<SelectListItem>();

            foreach (var item in Const.SEARCH_TYPE_CODE_LABEL)
            {
                selectList.Add(new SelectListItem { Text = item.Value, Value = item.Key });
            }

            return selectList;
        }

        /// <summary>
        /// 前／後工程区分のSelectListItem
        /// </summary>
        /// <param></param>
        /// <returns></returns>
        public static IList<SelectListItem> GetWorkTypeSelectList()
        {
            IList<SelectListItem> selectList = new List<SelectListItem>();

            foreach (var item in Const.WORK_TYPE_CODE_LABEL)
            {
                selectList.Add(new SelectListItem { Text = item.Value, Value = item.Key });
            }

            return selectList;
        }

        /// <summary>
        /// ステータス範囲区分のSelectListItem
        /// </summary>
        /// <param></param>
        /// <returns></returns>
        public static IList<SelectListItem> GetRangeTypeSelectList()
        {
            IList<SelectListItem> selectList = new List<SelectListItem>();

            foreach (var item in Const.RANGE_TYPE_CODE_LABEL)
            {
                selectList.Add(new SelectListItem { Text = item.Value, Value = item.Key });
            }

            return selectList;
        }

        public static bool IsValidForEnumHelper(Type type)
        {
            bool isValid = false;
            if (type != null)
            {
                // Type.IsEnum is false for Nullable<T> even if T is an enum.  Check underlying type (if any).
                // Do not support Enum type itself -- IsEnum property is false for that class.
                Type checkedType = Nullable.GetUnderlyingType(type) ?? type;
                if (checkedType.IsEnum)
                {
                    isValid = !HasFlagsInternal(checkedType);
                }
            }

            return isValid;
        }

        private static bool HasFlagsInternal(Type type)
        {
            if (type == null) throw new ArgumentNullException("type");

            FlagsAttribute attribute = type.GetCustomAttribute<FlagsAttribute>(inherit: false);
            return attribute != null;
        }

        public static bool IsValidForEnumHelper(ModelMetadata metadata)
        {
            return metadata != null && IsValidForEnumHelper(metadata.ModelType);
        }

        private static string GetDisplayName(FieldInfo field)
        {
            DisplayAttribute display = field.GetCustomAttribute<DisplayAttribute>(inherit: false);
            if (display != null)
            {
                string name = display.GetName();
                if (!String.IsNullOrEmpty(name))
                {
                    return name;
                }
            }

            return field.Name;
        }
    }
}